const resume = async (pool, readModelName, next) => {
  const {
    schemaName,
    escapeId,
    escapeStr,
    inlineLedgerForceStop,
    inlineLedgerExecuteStatement,
    PassthroughError
  } = pool;
  const databaseNameAsId = escapeId(schemaName);
  const ledgerTableNameAsId = escapeId(`__${schemaName}__LEDGER__`);

  while (true) {
    try {
      await inlineLedgerForceStop(pool, readModelName);
      await inlineLedgerExecuteStatement(pool, `
        WITH "CTE" AS (
         SELECT * FROM ${databaseNameAsId}.${ledgerTableNameAsId}
         WHERE "EventSubscriber" = ${escapeStr(readModelName)}
         FOR NO KEY UPDATE NOWAIT
       )
        UPDATE ${databaseNameAsId}.${ledgerTableNameAsId}
        SET "IsPaused" = FALSE
        WHERE "EventSubscriber" = ${escapeStr(readModelName)}
        AND (SELECT Count("CTE".*) FROM "CTE") = 1
      `);
      break;
    } catch (err) {
      if (!(err instanceof PassthroughError)) {
        throw err;
      }
    }
  }

  await next();
};

export default resume;
//# sourceMappingURL=resume.js.map