/// <reference types="node" />
import type { ObjectFixedIntersectionToObject, ObjectFunctionLikeKeys, CommonAdapterPool, CommonAdapterOptions, AdapterOperations, AdapterConnection, AdapterImplementation, StoreApi, PerformanceTracerLike, JsonMap, SearchCondition, UpdateCondition, ObjectFixedKeys, OmitObject, JsonPrimitive, FunctionLike, UnPromise } from 'resolve-readmodel-base';
import type RDSDataService from 'aws-sdk/clients/rdsdataservice';
import type crypto from 'crypto';
export * from 'resolve-readmodel-base';
export declare type LibDependencies = {
    RDSDataService: typeof RDSDataService;
    crypto: typeof crypto;
};
export declare type InlineLedgerExecuteStatementMethod = (pool: AdapterPool, querySQL: string, transactionId?: string | null, passthroughRuntimeErrors?: boolean) => Promise<Array<object>>;
export declare type ExecuteStatementMethod = (pool: AdapterPool, querySQL: string) => Promise<Array<object>>;
export declare type FullJitterMethod = (retries: number) => Promise<void>;
export declare type MakeNestedPathMethod = (nestedPath: Array<string>) => string;
export declare type EscapeableMethod = (str: string) => string;
export declare type InlineLedgerForceStopMethod = (pool: AdapterPool, readModelName: string) => Promise<void>;
export declare type BuildUpsertDocumentMethod = (searchExpression: Parameters<StoreApi<CommonAdapterPool>['update']>[3], updateExpression: Parameters<StoreApi<CommonAdapterPool>['update']>[4]) => JsonMap;
export declare type RowLike = JsonMap;
export declare type MarshalledRowLike = RowLike & {
    marshalled: 'marshalled';
};
export declare type ConvertResultRowMethod = (inputRow: MarshalledRowLike, fieldList: Parameters<StoreApi<CommonAdapterPool>['find']>[4] | null) => RowLike;
export declare type SearchToWhereExpressionMethod = (expression: SearchCondition, escapeId: EscapeableMethod, escapeStr: EscapeableMethod, makeNestedPath: MakeNestedPathMethod) => string;
export declare type UpdateToSetExpressionMethod = (expression: UpdateCondition, escapeId: EscapeableMethod, escapeStr: EscapeableMethod, makeNestedPath: MakeNestedPathMethod) => string;
export interface PassthroughErrorInstance extends Error {
    name: string;
    lastTransactionId: string | null | undefined;
}
export declare type PassthroughErrorFactory = {
    new (lastTransactionId: string | null | undefined): PassthroughErrorInstance;
} & {
    isPassthroughError: (error: Error & {
        code: string | number;
        stack: string;
    }, includeRuntimeErrors: boolean) => boolean;
};
export declare type IsRdsServiceErrorMethod = (error: Error & {
    code: string | number;
    stack: string;
}) => boolean;
export declare type GenerateGuidMethod = (...args: any) => string;
export declare type DropReadModelMethod = (pool: AdapterPool, readModelName: string) => Promise<void>;
export declare type HighloadMethodParameters<KS extends ObjectFunctionLikeKeys<InstanceType<LibDependencies['RDSDataService']>>, T extends {
    [K in KS]: FunctionLike;
}> = T[KS] extends {
    (params: infer Params, callback: infer Callback): infer Result;
    (callback: infer _Callback): infer _Result;
} ? Params : never;
export declare type HighloadMethodReturnType<KS extends ObjectFunctionLikeKeys<InstanceType<LibDependencies['RDSDataService']>>, T extends {
    [K in KS]: FunctionLike;
}> = Promise<T[KS] extends {
    (params: infer Params, callback: infer Callback): infer Result;
    (callback: infer _Callback): infer _Result;
} ? Result extends {
    promise: infer F;
} ? F extends FunctionLike ? UnPromise<ReturnType<F>> : never : never : never>;
export declare type WrapHighloadMethod = <KS extends ObjectFunctionLikeKeys<InstanceType<LibDependencies['RDSDataService']>>, T extends {
    [K in KS]: FunctionLike;
}>(isHighloadError: IsRdsServiceErrorMethod, obj: T, method: KS, params: HighloadMethodParameters<KS, T>) => HighloadMethodReturnType<KS, T>;
export declare type HighloadRdsMethod<KS extends ObjectFunctionLikeKeys<InstanceType<LibDependencies['RDSDataService']>>> = (...args: [
    HighloadMethodParameters<KS, InstanceType<LibDependencies['RDSDataService']>>
]) => HighloadMethodReturnType<KS, InstanceType<LibDependencies['RDSDataService']>>;
export declare type HighloadRdsDataService = {
    executeStatement: HighloadRdsMethod<'executeStatement'>;
    beginTransaction: HighloadRdsMethod<'beginTransaction'>;
    commitTransaction: HighloadRdsMethod<'commitTransaction'>;
    rollbackTransaction: HighloadRdsMethod<'rollbackTransaction'>;
};
export declare type AdapterOptions = CommonAdapterOptions & {
    dbClusterOrInstanceArn: RDSDataService.Arn;
    awsSecretStoreArn: RDSDataService.Arn;
    databaseName: RDSDataService.DbName;
    tablePrefix?: string;
} & RDSDataService.ClientConfiguration;
export declare type InternalMethods = {
    inlineLedgerExecuteStatement: InlineLedgerExecuteStatementMethod;
    executeStatement: ExecuteStatementMethod;
    inlineLedgerForceStop: InlineLedgerForceStopMethod;
    buildUpsertDocument: BuildUpsertDocumentMethod;
    isHighloadError: IsRdsServiceErrorMethod;
    isTimeoutError: IsRdsServiceErrorMethod;
    convertResultRow: ConvertResultRowMethod;
    searchToWhereExpression: SearchToWhereExpressionMethod;
    updateToSetExpression: UpdateToSetExpressionMethod;
    PassthroughError: PassthroughErrorFactory;
    generateGuid: GenerateGuidMethod;
    dropReadModel: DropReadModelMethod;
    escapeId: EscapeableMethod;
    escapeStr: EscapeableMethod;
    coercer: CoercerMethod;
};
export declare type ArrayOrSingleOrNull<T> = Array<T> | T | null;
export declare type UpdateFieldDescriptor = {
    [K in ObjectFixedKeys<ObjectFixedIntersectionToObject<UpdateCondition>>]: ObjectFixedIntersectionToObject<UpdateCondition>[K][string] | null;
} & {
    key: string;
    nestedKey: string[];
    baseName: string;
    selectedOperation: ObjectFixedKeys<UpdateCondition> | null;
    children: Map<string, UpdateFieldDescriptor>;
    operations?: ArrayOrSingleOrNull<{
        operationName: ObjectFixedKeys<UpdateCondition> | null;
        fieldValue: UpdateFieldDescriptor[ObjectFixedKeys<UpdateCondition>];
        nestedPath?: string[];
    }>;
};
export declare type CoercerMethod = (value: {
    intValue?: number;
    stringValue?: string;
    bigIntValue?: number;
    longValue?: number;
    booleanValue?: boolean;
    isNull?: boolean;
}) => JsonPrimitive;
export declare type AdapterPool = CommonAdapterPool & {
    hash512: (str: string) => string;
    performanceTracer: PerformanceTracerLike;
    makeNestedPath: MakeNestedPathMethod;
    rdsDataService: HighloadRdsDataService;
    dbClusterOrInstanceArn: RDSDataService.Arn;
    awsSecretStoreArn: RDSDataService.Arn;
    schemaName: RDSDataService.DbName;
    tablePrefix: string;
    xaTransactionId?: string | null | undefined;
    transactionId?: string | null | undefined;
} & {
    [K in keyof AdapterOperations<CommonAdapterPool>]: AdapterOperations<AdapterPool>[K];
} & {
    [K in keyof StoreApi<CommonAdapterPool>]: StoreApi<AdapterPool>[K];
} & InternalMethods;
export declare type CurrentAdapterConnection = AdapterConnection<AdapterPool, OmitObject<AdapterOptions, CommonAdapterPool>>;
export declare type ExternalMethods = {
    [K in keyof AdapterOperations<CommonAdapterPool>]: AdapterPool[K];
};
export declare type CurrentStoreApi = {
    [K in keyof StoreApi<CommonAdapterPool>]: AdapterPool[K];
};
export declare type ConnectionDependencies = LibDependencies & InternalMethods & ExternalMethods & CurrentStoreApi;
export declare type CurrentConnectMethod = (imports: ConnectionDependencies, ...args: Parameters<CurrentAdapterConnection['connect']>) => ReturnType<CurrentAdapterConnection['connect']>;
export declare type CurrentDisconnectMethod = CurrentAdapterConnection['disconnect'];
export declare type CurrentAdapterImplementation = AdapterImplementation<AdapterPool, AdapterOptions>;
export declare type AdminOptions = {
    awsSecretStoreAdminArn: AdapterOptions['awsSecretStoreArn'];
    dbClusterOrInstanceArn: AdapterOptions['dbClusterOrInstanceArn'];
    databaseName: AdapterOptions['databaseName'];
    region: AdapterOptions['region'];
    userLogin: string;
};
export declare type BoundResourceMethod = (options: AdminOptions) => Promise<void>;
export declare type UnboundResourceMethod = (pool: AdminPool, options: AdminOptions) => Promise<void>;
export declare type AdminPool = {
    connect: CurrentAdapterImplementation['connect'];
    disconnect: CurrentAdapterImplementation['disconnect'];
    escapeStr: EscapeableMethod;
    escapeId: EscapeableMethod;
    createResource: BoundResourceMethod;
    disposeResource: BoundResourceMethod;
    destroyResource: BoundResourceMethod;
};
